@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -36,7 +36,7 @@
 #include "enna.h"
 #include "enna_config.h"
 #include "browser.h"
-#include "view_cover.h"
+#include "box.h"
 #include "view_list.h"
 #include "view_wall.h"
 #include "image.h"
@@ -149,13 +149,13 @@ _browser_view_list_add(Smart_Data *sd)
 
 
 static Evas_Object *
-_browser_view_cover_add(Smart_Data *sd)
+_browser_box_add(Smart_Data *sd)
 {
     Evas_Object *view;
 
     if (!sd) return NULL;
 
-    view = enna_view_cover_add(enna->evas, 1);
+    view = enna_box_add(enna->layout, NULL);
 
     elm_layout_content_set(sd->layout, "enna.swallow.content", view);
     evas_object_smart_callback_add(view, "hilight", _view_hilight_cb, sd);
@@ -196,7 +196,7 @@ enna_browser_add(Evas * evas)
     return evas_object_smart_add(evas, _smart);
 }
 
-static int
+static Eina_Bool
 _view_delay_hilight_cb(void *data)
 {
     Browse_Data *bd = data;
@@ -209,7 +209,7 @@ _view_delay_hilight_cb(void *data)
     bd->sd->hilight_timer = NULL;
     evas_object_smart_callback_call (bd->sd->obj, "delay,hilight", ev);
 
-    return 0;
+    return EINA_FALSE;
 }
 
 static void
@@ -247,15 +247,15 @@ enna_browser_view_add(Evas_Object *obj, Enna_Browser_View_Type view_type)
         sd->view_funcs.view_files_get = enna_list_files_get;
         sd->view_funcs.view_jump_ascii = enna_list_jump_ascii;
         break;
-    case ENNA_BROWSER_VIEW_COVER:
-        sd->view_funcs.view_add = _browser_view_cover_add;
-        sd->view_funcs.view_append = enna_view_cover_file_append;
-        sd->view_funcs.view_selected_data_get = enna_view_cover_selected_data_get;
-        sd->view_funcs.view_jump_label = enna_view_cover_jump_label;
-        sd->view_funcs.view_key_down = enna_view_cover_input_feed;
-        sd->view_funcs.view_select_nth = enna_view_cover_select_nth;
-        sd->view_funcs.view_files_get = enna_view_cover_files_get;
-        sd->view_funcs.view_jump_ascii = enna_view_cover_jump_ascii;
+    case ENNA_BROWSER_BOX:
+        sd->view_funcs.view_add = _browser_box_add;
+        sd->view_funcs.view_append = enna_box_file_append;
+        sd->view_funcs.view_selected_data_get = enna_box_selected_data_get;
+        sd->view_funcs.view_jump_label = enna_box_jump_label;
+        sd->view_funcs.view_key_down = enna_box_input_feed;
+        sd->view_funcs.view_select_nth = enna_box_select_nth;
+        sd->view_funcs.view_files_get = enna_box_files_get;
+        sd->view_funcs.view_jump_ascii = enna_box_jump_ascii;
         break;
     case ENNA_BROWSER_VIEW_WALL:
         sd->view_funcs.view_add = _browser_view_wall_add;
@@ -546,7 +546,7 @@ _browse(void *data)
            if (!eina_list_count(sd->files))
             {
 
-                sd->file = enna_vfs_create_directory(sd->file->uri, _("No media found !"), "icon/nofile", NULL);
+                sd->file = enna_vfs_create_directory(sd->file->uri, _("No media found!"), "icon/nofile", NULL);
                 sd->files = NULL;
                 sd->files = eina_list_append(sd->files,sd->file);
 
@@ -561,12 +561,6 @@ _browse(void *data)
         else if (sd->show_file)
         {
             /* File selected is a regular file */
-            Enna_Vfs_File *prev_vfs;
-            char *prev_uri;
-            prev_vfs = sd->vfs->func.class_vfs_get(sd->vfs->cookie);
-                prev_uri = prev_vfs->uri ? strdup(prev_vfs->uri) : NULL;
-            sd->files = sd->vfs->func.class_browse_up(prev_uri, sd->vfs->cookie);
-            ENNA_FREE(prev_uri);
             ev->files = sd->files;
             sd->accept_ev = 1;
             evas_object_smart_callback_call (sd->obj, "selected", ev);
@@ -624,6 +618,9 @@ _list_transition_core(Smart_Data *sd, unsigned char direction)
     Eina_List *l;
     Eina_List *files = sd->files;
 
+    if (!files)
+      return;
+
     if (!direction)
     {
         edje_object_signal_callback_del(elm_layout_edje_get(sd->layout),
@@ -706,7 +703,7 @@ _list_transition_core(Smart_Data *sd, unsigned char direction)
     sd->o_parent = elm_button_add(enna->layout);
     elm_object_style_set(sd->o_parent, "simple");
     evas_object_smart_callback_add(sd->o_parent, "clicked", _parent_clicked_cb, sd);
-    elm_button_label_set(sd->o_parent, sd->file ? sd->file->label : _("Main Menu"));
+    elm_button_label_set(sd->o_parent, sd->file ? sd->file->label : _("Main menu"));
     ic = elm_icon_add(enna->layout);
     if (!sd->file)
         elm_icon_file_set(ic, enna_config_theme_get(), "icon/home");
@@ -737,7 +734,7 @@ _list_transition_right_end_cb(void *data, Evas_Object *o, const char *sig, const
     _list_transition_core(data, 1);
 }
 
-static int
+static Eina_Bool
 _letter_timer_cb(void *data)
 {
     Smart_Data *sd;
@@ -823,7 +820,7 @@ enna_browser_input_feed(Evas_Object *obj, enna_input event)
 
     switch (event)
     {
-    case ENNA_INPUT_EXIT:
+    case ENNA_INPUT_BACK:
         _browse_down(sd);
         break;
     case ENNA_INPUT_OK:
@@ -838,8 +835,8 @@ enna_browser_input_feed(Evas_Object *obj, enna_input event)
     case ENNA_INPUT_LEFT:
     case ENNA_INPUT_NEXT:
     case ENNA_INPUT_PREV:
-    case ENNA_INPUT_HOME:
-    case ENNA_INPUT_END:
+    case ENNA_INPUT_FIRST:
+    case ENNA_INPUT_LAST:
         if (sd->view_funcs.view_key_down)
             sd->view_funcs.view_key_down(sd->o_view, event);
         break;
