@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -22,9 +22,9 @@
 #ifndef EXIT_H
 #define EXIT_H
 
-Evas_Object *enna_exit_add(Evas * evas);
-void enna_exit_update_text(Evas_Object *obj);
-void enna_exit_show(Evas_Object *obj);
-void enna_exit_hide(Evas_Object *obj);
+int enna_exit_init();
+int enna_exit_shutdown();
 
+void enna_exit_update_text();
+Eina_Bool enna_exit_visible();
 #endif /* EXIT_H */
