@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -22,7 +22,7 @@
 #ifndef IPC_H
 #define IPC_H
 
-EAPI int enna_ipc_init(void);
-EAPI int enna_ipc_shutdown(void);
+int enna_ipc_init(void);
+int enna_ipc_shutdown(void);
 
 #endif
