@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -32,4 +32,19 @@ void enna_log_shutdown(void);
 #define enna_log(level,module,fmt,arg...) \
         enna_log_print(level,module,__FILE__,__LINE__,fmt,##arg)
 
+#define WRN(fmt,arg...) \
+	enna_log_print(ENNA_MSG_WARNING,"", __FILE__,__LINE__,fmt,##arg)
+
+#define ERR(fmt,arg...) \
+	enna_log_print(ENNA_MSG_ERROR,"", __FILE__,__LINE__,fmt,##arg)
+
+#define CRIT(fmt,arg...) \
+	enna_log_print(ENNA_MSG_CRITICAL,"", __FILE__,__LINE__,fmt,##arg)
+
+#define DBG(fmt,arg...) \
+	enna_log(ENNA_MSG_INFO,"",fmt,##arg)
+	
+#define EVT(fmt,arg...) \
+	enna_log_print(ENNA_MSG_EVENT,"", __FILE__,__LINE__,fmt,##arg)
+
 #endif /* LOGS_H */
