@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -23,7 +23,6 @@
 #include <string.h>
 
 #include <Ecore.h>
-#include <Ecore_Data.h>
 #include <Ecore_Input.h>
 
 #include <player.h>
@@ -70,6 +69,22 @@ typedef struct list_item_s
     const char *label;
 } list_item_t;
 
+typedef struct mediaplayer_cfg_s {
+    player_type_t type;
+    player_type_t dvd_type;
+    player_type_t tv_type;
+    player_vo_t vo;
+    player_ao_t ao;
+    player_verbosity_level_t verbosity;
+    int dvd_set;
+    int tv_set;
+    char *sub_align;
+    char *sub_pos;
+    char *sub_scale;
+    char *sub_visibility;
+    char *framedrop;
+} mediaplayer_cfg_t;
+
 typedef struct _Enna_Mediaplayer Enna_Mediaplayer;
 
 struct _Enna_Mediaplayer
@@ -77,7 +92,7 @@ struct _Enna_Mediaplayer
     PLAY_STATE play_state;
     player_t *player;
     player_t *players[MAX_PLAYERS];
-    void (*event_cb)(void *data, enna_mediaplayer_event_t event);
+    int (*event_cb)(enna_mediaplayer_event_t event, void *data);
     void *event_cb_data;
     char *uri;
     char *label;
@@ -99,10 +114,10 @@ struct _Enna_Mediaplayer
 };
 
 static Enna_Mediaplayer *mp = NULL;
-static Enna_Config_Video *config_video = NULL;
+static mediaplayer_cfg_t mp_cfg;
 
-static void
-_event_cb(void *data, enna_mediaplayer_event_t event)
+static int
+_event_cb(enna_mediaplayer_event_t event, void *data)
 {
     switch (event)
     {
@@ -113,6 +128,7 @@ _event_cb(void *data, enna_mediaplayer_event_t event)
     default:
         break;
     }
+    return 0;
 }
 
 static void
@@ -123,7 +139,7 @@ pipe_read(void *data, void *buf, unsigned int nbyte)
     if (!mp->event_cb || !buf)
         return;
 
-    mp->event_cb(mp->event_cb_data, *event);
+    mp->event_cb(*event, mp->event_cb_data);
 }
 
 static int
@@ -140,7 +156,7 @@ event_cb(player_event_t e, void *data)
     return 0;
 }
 
-static int
+static Eina_Bool
 event_key_down(void *data, int type, void *event)
 {
     Ecore_Event_Key *e;
@@ -163,15 +179,19 @@ event_key_down(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 event_mouse_button(void *data, int type, void *event)
 {
+    mrl_resource_t res;
     Ecore_Event_Mouse_Button *e = event;
 
     /* Broadcast mouse position only for dvd player and only
        if libplayer window is on screen */
-    if ((e->window == enna->ee_winid) || !mp->uri ||
-        strncmp(mp->uri, URI_TYPE_DVDNAV, strlen(URI_TYPE_DVDNAV)))
+    if ((e->window == enna->ee_winid) || !mp->uri)
+        return 1;
+
+    res = mrl_get_resource(mp->player, NULL);
+    if (res != MRL_RESOURCE_DVDNAV)
         return 1;
 
     /* Set mouse position and send mouseclick event */
@@ -182,15 +202,19 @@ event_mouse_button(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 event_mouse_move(void *data, int type, void *event)
 {
+    mrl_resource_t res;
     Ecore_Event_Mouse_Move *e = event;
 
     /* Broadcast mouse position only for dvd player and only
        if libplayer window is on screen */
-    if ((e->window == enna->ee_winid) || !mp->uri ||
-        strncmp(mp->uri, URI_TYPE_DVDNAV, strlen(URI_TYPE_DVDNAV)))
+    if ((e->window == enna->ee_winid) || !mp->uri)
+        return 1;
+
+    res = mrl_get_resource(mp->player, NULL);
+    if (res != MRL_RESOURCE_DVDNAV)
         return 1;
 
     /* Send mouse position to libplayer */
@@ -334,58 +358,53 @@ set_local_stream(const char *uri)
 static void
 init_sub_align(void)
 {
-    if (!config_video || !config_video->sub_align ||
-        !strcmp(config_video->sub_align, "auto"))
+    if (!mp_cfg.sub_align || !strcmp(mp_cfg.sub_align, "auto"))
         mp->subtitle_alignment = SUB_ALIGNMENT_DEFAULT;
-    else if (!strcmp(config_video->sub_align, "bottom"))
+    else if (!strcmp(mp_cfg.sub_align, "bottom"))
         mp->subtitle_alignment = PLAYER_SUB_ALIGNMENT_BOTTOM;
-    else if (!strcmp(config_video->sub_align, "middle"))
+    else if (!strcmp(mp_cfg.sub_align, "middle"))
         mp->subtitle_alignment = PLAYER_SUB_ALIGNMENT_CENTER;
-    else if (!strcmp(config_video->sub_align, "top"))
+    else if (!strcmp(mp_cfg.sub_align, "top"))
         mp->subtitle_alignment = PLAYER_SUB_ALIGNMENT_TOP;
 }
 
 static void
 init_sub_pos(void)
 {
-    if (!config_video || !config_video->sub_pos ||
-        !strcmp(config_video->sub_pos, "auto"))
+    if (!mp_cfg.sub_pos || !strcmp(mp_cfg.sub_pos, "auto"))
         mp->subtitle_position = SUB_POSITION_DEFAULT;
     else
-        mp->subtitle_position = atoi(config_video->sub_pos);
+        mp->subtitle_position = atoi(mp_cfg.sub_pos);
 }
 
 static void
 init_sub_scale(void)
 {
-    if (!config_video || !config_video->sub_scale ||
-        !strcmp(config_video->sub_scale, "auto"))
+    if (!mp_cfg.sub_scale || !strcmp(mp_cfg.sub_scale, "auto"))
         mp->subtitle_scale = SUB_SCALE_DEFAULT;
     else
-        mp->subtitle_scale = atoi(config_video->sub_scale);
+        mp->subtitle_scale = atoi(mp_cfg.sub_scale);
 }
 
 static void
 init_sub_visibility(void)
 {
-    if (!config_video || !config_video->sub_visibility ||
-        !strcmp(config_video->sub_visibility, "auto"))
+    if (!mp_cfg.sub_visibility || !strcmp(mp_cfg.sub_visibility, "auto"))
         mp->subtitle_visibility = SUB_VISIBILITY_DEFAULT;
-    else if (!strcmp(config_video->sub_visibility, "no"))
+    else if (!strcmp(mp_cfg.sub_visibility, "no"))
         mp->subtitle_visibility = 0;
-    else if (!strcmp(config_video->sub_visibility, "yes"))
+    else if (!strcmp(mp_cfg.sub_visibility, "yes"))
         mp->subtitle_visibility = 1;
 }
 
 static void
 init_framedrop(void)
 {
-    if (!config_video || !config_video->framedrop ||
-        !strcmp(config_video->framedrop, "no"))
+    if (!mp_cfg.framedrop || !strcmp(mp_cfg.framedrop, "no"))
         mp->framedrop = FRAMEDROP_DEFAULT;
-    else if (!strcmp(config_video->framedrop, "soft"))
+    else if (!strcmp(mp_cfg.framedrop, "soft"))
         mp->framedrop = PLAYER_FRAMEDROP_SOFT;
-    else if (!strcmp(config_video->framedrop, "hard"))
+    else if (!strcmp(mp_cfg.framedrop, "hard"))
         mp->framedrop = PLAYER_FRAMEDROP_HARD;
 }
 
@@ -461,7 +480,17 @@ mp_file_set(const char *uri, const char *label)
     }
     /* default is local files */
     if (!mrl)
-        mrl = set_local_stream(uri);
+    {
+        const char *it;
+        it = strrchr(uri, '.');
+        if (it && !strcmp(it, ".iso")) /* consider ISO file as DVD */
+        {
+            mrl = set_dvd_stream(uri, MRL_RESOURCE_DVDNAV);
+            player_type = mp->dvd_type;
+        }
+        else
+            mrl = set_local_stream(uri);
+    }
 
     if (!mrl)
         return 1;
@@ -475,9 +504,6 @@ mp_file_set(const char *uri, const char *label)
     mp->audio_delay = AUDIO_DELAY_DEFAULT;
     mp->subtitle_delay = SUB_DELAY_DEFAULT;
 
-    /* Get the video configuration parameters from file */
-    config_video = enna_config->cfg_video;
-
     /* Initialization of subtitles variables */
     init_sub_align();
     init_sub_pos();
@@ -569,8 +595,7 @@ mp_video_resize(int x, int y, int w, int h)
 }
 
 static void
-mp_event_cb_set(void (*event_cb)(void *data,
-                                 enna_mediaplayer_event_t event),
+mp_event_cb_set(int (*event_cb)(enna_mediaplayer_event_t event, void *data),
                 void *data)
 {
     /* FIXME: function to call when end of stream is send by libplayer */
@@ -587,14 +612,13 @@ mp_send_key(enna_input event)
         [ENNA_INPUT_RIGHT]      = PLAYER_VDR_RIGHT,
         [ENNA_INPUT_UP]         = PLAYER_VDR_UP,
         [ENNA_INPUT_DOWN]       = PLAYER_VDR_DOWN,
-        [ENNA_INPUT_HOME]       = PLAYER_VDR_PREVIOUS,
-        [ENNA_INPUT_END]        = PLAYER_VDR_NEXT,
-        [ENNA_INPUT_NEXT]       = PLAYER_VDR_CHANNELPLUS,
-        [ENNA_INPUT_PREV]       = PLAYER_VDR_CHANNELMINUS,
+        [ENNA_INPUT_CHANPLUS]   = PLAYER_VDR_CHANNELPLUS,
+        [ENNA_INPUT_CHANMINUS]  = PLAYER_VDR_CHANNELMINUS,
+        [ENNA_INPUT_PREV]       = PLAYER_VDR_PREVIOUS,
+        [ENNA_INPUT_NEXT]       = PLAYER_VDR_NEXT,
         [ENNA_INPUT_OK]         = PLAYER_VDR_OK,
-        [ENNA_INPUT_EXIT]       = PLAYER_VDR_BACK,
-        [ENNA_INPUT_QUIT]       = PLAYER_VDR_BACK,
-        [ENNA_INPUT_KEY_N]      = PLAYER_VDR_INFO,
+        [ENNA_INPUT_BACK]       = PLAYER_VDR_BACK,
+        [ENNA_INPUT_INFO]       = PLAYER_VDR_INFO,
         [ENNA_INPUT_KEY_0]      = PLAYER_VDR_0,
         [ENNA_INPUT_KEY_1]      = PLAYER_VDR_1,
         [ENNA_INPUT_KEY_2]      = PLAYER_VDR_2,
@@ -605,33 +629,29 @@ mp_send_key(enna_input event)
         [ENNA_INPUT_KEY_7]      = PLAYER_VDR_7,
         [ENNA_INPUT_KEY_8]      = PLAYER_VDR_8,
         [ENNA_INPUT_KEY_9]      = PLAYER_VDR_9,
-        [ENNA_INPUT_KEY_Q]      = PLAYER_VDR_RED,
-        [ENNA_INPUT_KEY_W]      = PLAYER_VDR_GREEN,
-        [ENNA_INPUT_KEY_E]      = PLAYER_VDR_YELLOW,
-        [ENNA_INPUT_KEY_R]      = PLAYER_VDR_BLUE,
-        [ENNA_INPUT_KEY_A]      = PLAYER_VDR_PLAY,
-        [ENNA_INPUT_KEY_S]      = PLAYER_VDR_PAUSE,
-        [ENNA_INPUT_KEY_D]      = PLAYER_VDR_STOP,
-        [ENNA_INPUT_KEY_F]      = PLAYER_VDR_RECORD,
-        [ENNA_INPUT_KEY_B]      = PLAYER_VDR_CHANNELPREVIOUS,
-        [ENNA_INPUT_KEY_G]      = PLAYER_VDR_USER_1,
-        [ENNA_INPUT_KEY_H]      = PLAYER_VDR_USER_2,
-        [ENNA_INPUT_KEY_J]      = PLAYER_VDR_USER_3,
-        [ENNA_INPUT_KEY_K]      = PLAYER_VDR_USER_4,
-        [ENNA_INPUT_KEY_L]      = PLAYER_VDR_USER_5,
-        [ENNA_INPUT_KEY_Z]      = PLAYER_VDR_FASTREW,
-        [ENNA_INPUT_KEY_X]      = PLAYER_VDR_FASTFWD,
-        [ENNA_INPUT_KEY_C]      = PLAYER_VDR_AUDIO,
-        [ENNA_INPUT_KEY_V]      = PLAYER_VDR_SUBTITLES,
-        [ENNA_INPUT_PLUS]       = PLAYER_VDR_VOLPLUS,
-        [ENNA_INPUT_MINUS]      = PLAYER_VDR_VOLMINUS,
-        [ENNA_INPUT_KEY_M]      = PLAYER_VDR_MUTE,
-        [ENNA_INPUT_KEY_T]      = PLAYER_VDR_SCHEDULE,
-        [ENNA_INPUT_KEY_Y]      = PLAYER_VDR_CHANNELS,
-        [ENNA_INPUT_KEY_U]      = PLAYER_VDR_TIMERS,
-        [ENNA_INPUT_KEY_I]      = PLAYER_VDR_RECORDINGS,
-        [ENNA_INPUT_KEY_O]      = PLAYER_VDR_SETUP,
-        [ENNA_INPUT_KEY_P]      = PLAYER_VDR_COMMANDS,
+        [ENNA_INPUT_RED]        = PLAYER_VDR_RED,
+        [ENNA_INPUT_GREEN]      = PLAYER_VDR_GREEN,
+        [ENNA_INPUT_YELLOW]     = PLAYER_VDR_YELLOW,
+        [ENNA_INPUT_BLUE]       = PLAYER_VDR_BLUE,
+        [ENNA_INPUT_PLAY]       = PLAYER_VDR_PLAY,
+        [ENNA_INPUT_PAUSE]      = PLAYER_VDR_PAUSE,
+        [ENNA_INPUT_STOP]       = PLAYER_VDR_STOP,
+        [ENNA_INPUT_RECORD]     = PLAYER_VDR_RECORD,
+        [ENNA_INPUT_CHANPREV]   = PLAYER_VDR_CHANNELPREVIOUS,
+        [ENNA_INPUT_FORWARD]    = PLAYER_VDR_FASTREW,
+        [ENNA_INPUT_REWIND]     = PLAYER_VDR_FASTFWD,
+        [ENNA_INPUT_AUDIO_PREV] = PLAYER_VDR_AUDIO,
+        [ENNA_INPUT_AUDIO_NEXT] = PLAYER_VDR_AUDIO,
+        [ENNA_INPUT_SUBTITLES]  = PLAYER_VDR_SUBTITLES,
+        [ENNA_INPUT_SUBS_PREV]  = PLAYER_VDR_SUBTITLES,
+        [ENNA_INPUT_SUBS_NEXT]  = PLAYER_VDR_SUBTITLES,
+        [ENNA_INPUT_VOLPLUS]    = PLAYER_VDR_VOLPLUS,
+        [ENNA_INPUT_VOLMINUS]   = PLAYER_VDR_VOLMINUS,
+        [ENNA_INPUT_MUTE]       = PLAYER_VDR_MUTE,
+        [ENNA_INPUT_SCHEDULE]   = PLAYER_VDR_SCHEDULE,
+        [ENNA_INPUT_CHANNELS]   = PLAYER_VDR_CHANNELS,
+        [ENNA_INPUT_TIMERS]     = PLAYER_VDR_TIMERS,
+        [ENNA_INPUT_RECORDINGS] = PLAYER_VDR_RECORDINGS,
     };
 
     if (mp->uri && strncmp(mp->uri, URI_TYPE_VDR, strlen(URI_TYPE_VDR)) &&
@@ -682,190 +702,279 @@ enna_mediaplayer_supported_uri_type(enna_mediaplayer_uri_type_t type)
                                            type_list[type].res);
 }
 
-#define ENNA_MP_PLAYER_INIT(type)                                            \
-    do                                                                       \
-    {                                                                        \
-        mp->players[type] =                                                  \
-            player_init(type, ao, vo, verbosity, enna->ee_winid, event_cb);  \
-        if (!mp->players[type])                                              \
-            goto err;                                                        \
-    }                                                                        \
-    while(0)
+#define CFG_VIDEO(field)                                                \
+    value = enna_config_string_get(section, #field);                    \
+    mp_cfg.field = value ? strdup(value) : NULL;
 
-#define CFG_VIDEO(field) \
-    enna_config_value_store(&video->field, #field, ENNA_CONFIG_STRING, pair);
+static const struct {
+    const char *name;
+    player_type_t type;
+} map_player_type[] = {
+    { "xine",       PLAYER_TYPE_XINE         },
+    { "mplayer",    PLAYER_TYPE_MPLAYER      },
+    { "vlc",        PLAYER_TYPE_VLC          },
+    { "gstreamer",  PLAYER_TYPE_GSTREAMER    },
+    { NULL,         PLAYER_TYPE_DUMMY        }
+};
+
+static const struct {
+    const char *name;
+    player_vo_t vo;
+} map_player_vo[] = {
+    { "auto",       PLAYER_VO_AUTO           },
+    { "vdpau",      PLAYER_VO_VDPAU          },
+    { "x11",        PLAYER_VO_X11            },
+    { "xv",         PLAYER_VO_XV             },
+    { "gl",         PLAYER_VO_GL             },
+    { "fb",         PLAYER_VO_FB             },
+    { NULL,         PLAYER_VO_NULL           }
+};
+
+static const struct {
+    const char *name;
+    player_ao_t ao;
+} map_player_ao[] = {
+    { "auto",       PLAYER_AO_AUTO           },
+    { "alsa",       PLAYER_AO_ALSA           },
+    { "oss",        PLAYER_AO_OSS            },
+    { "pulse",      PLAYER_AO_PULSE          },
+    { NULL,         PLAYER_AO_NULL           }
+};
+
+static const struct {
+    const char *name;
+    player_verbosity_level_t verb;
+} map_player_verbosity[] = {
+    { "none",        PLAYER_MSG_NONE         },
+    { "verbose",     PLAYER_MSG_VERBOSE      },
+    { "info",        PLAYER_MSG_INFO         },
+    { "warning",     PLAYER_MSG_WARNING      },
+    { "error",       PLAYER_MSG_ERROR        },
+    { "critical",    PLAYER_MSG_CRITICAL     },
+    { NULL,          PLAYER_MSG_NONE         }
+};
+
+static void
+cfg_mediaplayer_section_load (const char *section)
+{
+    const char *value = NULL;
+    int i;
+
+    enna_log(ENNA_MSG_INFO, NULL, "parameters:");
+
+    value = enna_config_string_get(section, "type");
+    if (value)
+    {
+        enna_log(ENNA_MSG_INFO, NULL, " * type: %s", value);
+
+        for (i = 0; map_player_type[i].name; i++)
+            if (!strcmp(value, map_player_type[i].name))
+            {
+                mp_cfg.type = map_player_type[i].type;
+                break;
+            }
+    }
+
+    value = enna_config_string_get(section, "dvd_type");
+    if (value)
+    {
+        enna_log(ENNA_MSG_INFO, NULL, " * dvd_type: %s", value);
+
+        for (i = 0; map_player_type[i].name; i++)
+            if (!strcmp(value, map_player_type[i].name))
+            {
+                mp_cfg.dvd_type = map_player_type[i].type;
+                break;
+            }
+        mp_cfg.dvd_set = 1;
+    }
+
+    value = enna_config_string_get(section, "tv_type");
+    if (value)
+    {
+        enna_log(ENNA_MSG_INFO, NULL, " * tv_type: %s", value);
+
+        for (i = 0; map_player_type[i].name; i++)
+            if (!strcmp(value, map_player_type[i].name))
+            {
+                mp_cfg.tv_type = map_player_type[i].type;
+                break;
+            }
+        mp_cfg.tv_set = 1;
+    }
+
+    value = enna_config_string_get(section, "video_out");
+    if (value)
+    {
+        enna_log(ENNA_MSG_INFO, NULL, " * video out: %s", value);
+
+        for (i = 0; map_player_vo[i].name; i++)
+            if (!strcmp(value, map_player_vo[i].name))
+            {
+                mp_cfg.vo = map_player_vo[i].vo;
+                break;
+            }
+    }
+
+    value = enna_config_string_get(section, "audio_out");
+    if (value)
+    {
+        enna_log(ENNA_MSG_INFO, NULL, " * audio out: %s", value);
+
+        for (i = 0; map_player_ao[i].name; i++)
+            if (!strcmp(value, map_player_ao[i].name))
+            {
+                mp_cfg.ao = map_player_ao[i].ao;
+                break;
+            }
+    }
+
+    value = enna_config_string_get(section, "verbosity");
+    if (value)
+    {
+        enna_log(ENNA_MSG_INFO, NULL, " * verbosity level: %s", value);
+
+        for (i = 0; map_player_verbosity[i].name; i++)
+            if (!strcmp(value, map_player_verbosity[i].name))
+            {
+                mp_cfg.verbosity = map_player_verbosity[i].verb;
+                break;
+            }
+    }
+
+    CFG_VIDEO(sub_align);
+    CFG_VIDEO(sub_pos);
+    CFG_VIDEO(sub_scale);
+    CFG_VIDEO(sub_visibility);
+    CFG_VIDEO(framedrop);
+
+    if (!value)
+        enna_log(ENNA_MSG_INFO, NULL, " * use all parameters by default");
+}
+
+static void
+cfg_mediaplayer_section_save (const char *section)
+{
+    int i;
+
+    /* Default type */
+    for (i = 0; map_player_type[i].name; i++)
+        if (mp_cfg.type == map_player_type[i].type)
+        {
+            enna_config_string_set(section, "type",
+                                   map_player_type[i].name);
+            break;
+        }
+
+    /* DVD Type */
+    for (i = 0; map_player_type[i].name; i++)
+        if (mp_cfg.dvd_type == map_player_type[i].type)
+        {
+            enna_config_string_set(section, "dvd_type",
+                                   map_player_type[i].name);
+            break;
+        }
+
+    /* TV Type */
+    for (i = 0; map_player_type[i].name; i++)
+        if (mp_cfg.tv_type == map_player_type[i].type)
+        {
+            enna_config_string_set(section, "tv_type",
+                                   map_player_type[i].name);
+            break;
+        }
+
+    /* VO Type */
+    for (i = 0; map_player_vo[i].name; i++)
+        if (mp_cfg.vo == map_player_vo[i].vo)
+        {
+            enna_config_string_set(section, "video_out",
+                                   map_player_vo[i].name);
+            break;
+        }
+
+    /* AO Type */
+    for (i = 0; map_player_ao[i].name; i++)
+        if (mp_cfg.ao == map_player_ao[i].ao)
+        {
+            enna_config_string_set(section, "audio_out",
+                                   map_player_ao[i].name);
+            break;
+        }
+
+    /* Verbosity */
+    for (i = 0; map_player_verbosity[i].name; i++)
+        if (mp_cfg.verbosity == map_player_verbosity[i].verb)
+        {
+            enna_config_string_set(section, "verbosity",
+                                   map_player_verbosity[i].name);
+            break;
+        }
+
+    enna_config_string_set(section, "sub_align",      mp_cfg.sub_align);
+    enna_config_string_set(section, "sub_pos",        mp_cfg.sub_pos);
+    enna_config_string_set(section, "sub_scale",      mp_cfg.sub_scale);
+    enna_config_string_set(section, "sub_visibility", mp_cfg.sub_visibility);
+    enna_config_string_set(section, "framedrop",      mp_cfg.framedrop);
+}
+
+static void
+cfg_mediaplayer_free (void)
+{
+    ENNA_FREE(mp_cfg.sub_align);
+    ENNA_FREE(mp_cfg.sub_pos);
+    ENNA_FREE(mp_cfg.sub_scale);
+    ENNA_FREE(mp_cfg.sub_visibility);
+    ENNA_FREE(mp_cfg.framedrop);
+}
+
+static void
+cfg_mediaplayer_section_set_default (void)
+{
+    cfg_mediaplayer_free();
+
+    mp_cfg.type           = PLAYER_TYPE_MPLAYER;
+    mp_cfg.dvd_type       = PLAYER_TYPE_XINE;
+    mp_cfg.tv_type        = PLAYER_TYPE_XINE;
+    mp_cfg.vo             = PLAYER_VO_AUTO;
+    mp_cfg.ao             = PLAYER_AO_AUTO;
+    mp_cfg.verbosity      = PLAYER_MSG_WARNING;
+    mp_cfg.dvd_set        = 0;
+    mp_cfg.tv_set         = 0;
+    mp_cfg.sub_align      = NULL;
+    mp_cfg.sub_pos        = NULL;
+    mp_cfg.sub_scale      = NULL;
+    mp_cfg.sub_visibility = NULL;
+    mp_cfg.framedrop      = NULL;
+}
+
+static Enna_Config_Section_Parser cfg_mediaplayer = {
+    "mediaplayer",
+    cfg_mediaplayer_section_load,
+    cfg_mediaplayer_section_save,
+    cfg_mediaplayer_section_set_default,
+    cfg_mediaplayer_free,
+};
+
+void
+enna_mediaplayer_cfg_register (void)
+{
+    enna_config_section_parser_register(&cfg_mediaplayer);
+}
 
 int
 enna_mediaplayer_init(void)
 {
-    Enna_Config_Data *cfgdata;
-    Enna_Config_Video *video;
-    char *value = NULL;
+    player_init_param_t param;
 
-    player_type_t type = PLAYER_TYPE_MPLAYER;
-    player_type_t dvd_type = PLAYER_TYPE_XINE;
-    player_type_t tv_type = PLAYER_TYPE_XINE;
-    player_vo_t vo = PLAYER_VO_AUTO;
-    player_ao_t ao = PLAYER_AO_AUTO;
-    player_verbosity_level_t verbosity = PLAYER_MSG_WARNING;
-
-    int dvd_set = 0, tv_set = 0;
-
-    /* set default video config */
-    enna_config->cfg_video = calloc(1, sizeof (Enna_Config_Video));
-    video = enna_config->cfg_video;
-
-    /* Load Config file values */
-    cfgdata = enna_config_module_pair_get("mediaplayer");
-
-    enna_log(ENNA_MSG_INFO, NULL, "parameters:");
-
-    if (cfgdata)
-    {
-        Eina_List *l;
-
-        for (l = cfgdata->pair; l; l = l->next)
-        {
-            Config_Pair *pair = l->data;
-
-            if (!strcmp("type", pair->key))
-            {
-                enna_config_value_store(&value, "type",
-                                        ENNA_CONFIG_STRING, pair);
-                enna_log(ENNA_MSG_INFO, NULL,
-                         " * type: %s", value);
-
-                if (!strcmp("gstreamer", value))
-                    type = PLAYER_TYPE_GSTREAMER;
-                else if (!strcmp("mplayer", value))
-                    type = PLAYER_TYPE_MPLAYER;
-                else if (!strcmp("vlc", value))
-                    type = PLAYER_TYPE_VLC;
-                else if (!strcmp("xine", value))
-                    type = PLAYER_TYPE_XINE;
-                else
-                    enna_log(ENNA_MSG_WARNING, NULL,
-                             "   - unknown type, 'mplayer' used instead");
-            }
-            else if (!strcmp("dvd_type", pair->key))
-            {
-                enna_config_value_store(&value, "dvd_type",
-                                        ENNA_CONFIG_STRING, pair);
-                enna_log(ENNA_MSG_INFO, NULL,
-                         " * dvd_type: %s", value);
-
-                if (!strcmp("gstreamer", value))
-                    dvd_type = PLAYER_TYPE_GSTREAMER;
-                else if (!strcmp("mplayer", value))
-                    dvd_type = PLAYER_TYPE_MPLAYER;
-                else if (!strcmp("vlc", value))
-                    dvd_type = PLAYER_TYPE_VLC;
-                else if (!strcmp("xine", value))
-                    dvd_type = PLAYER_TYPE_XINE;
-                else
-                    enna_log(ENNA_MSG_WARNING, NULL,
-                             "   - unknown dvd_type, 'xine' used instead");
-                dvd_set = 1;
-            }
-            else if (!strcmp("tv_type", pair->key))
-            {
-                enna_config_value_store(&value, "tv_type",
-                                        ENNA_CONFIG_STRING, pair);
-                enna_log(ENNA_MSG_INFO, NULL,
-                         " * tv_type: %s", value);
-
-                if (!strcmp("gstreamer", value))
-                    tv_type = PLAYER_TYPE_GSTREAMER;
-                else if (!strcmp("mplayer", value))
-                    tv_type = PLAYER_TYPE_MPLAYER;
-                else if (!strcmp("vlc", value))
-                    tv_type = PLAYER_TYPE_VLC;
-                else if (!strcmp("xine", value))
-                    tv_type = PLAYER_TYPE_XINE;
-                else
-                    enna_log(ENNA_MSG_WARNING, NULL,
-                             "   - unknown tv_type, 'xine' used instead");
-                tv_set = 1;
-            }
-            else if (!strcmp("video_out", pair->key))
-            {
-                enna_config_value_store(&value, "video_out",
-                                        ENNA_CONFIG_STRING, pair);
-                enna_log(ENNA_MSG_INFO, NULL,
-                         " * video out: %s", value);
-
-                if (!strcmp("auto", value))
-                    vo = PLAYER_VO_AUTO;
-                else if (!strcmp("vdpau", value))
-                    vo = PLAYER_VO_VDPAU;
-                else if (!strcmp("x11", value))
-                    vo = PLAYER_VO_X11;
-                else if (!strcmp("xv", value))
-                    vo = PLAYER_VO_XV;
-                else if (!strcmp("gl", value))
-                    vo = PLAYER_VO_GL;
-                else if (!strcmp("fb", value))
-                    vo = PLAYER_VO_FB;
-                else
-                    enna_log(ENNA_MSG_WARNING, NULL,
-                             "   - unknown video_out, 'auto' used instead");
-            }
-            else if (!strcmp("audio_out", pair->key))
-            {
-                enna_config_value_store(&value, "audio_out",
-                                        ENNA_CONFIG_STRING, pair);
-                enna_log(ENNA_MSG_INFO, NULL,
-                         " * audio out: %s", value);
-
-                if (!strcmp("auto", value))
-                    ao = PLAYER_AO_AUTO;
-                else if (!strcmp("alsa", value))
-                    ao = PLAYER_AO_ALSA;
-                else if (!strcmp("oss", value))
-                    ao = PLAYER_AO_OSS;
-                else
-                    enna_log(ENNA_MSG_WARNING, NULL,
-                             "   - unknown audio_out, 'auto' used instead");
-            }
-            else if (!strcmp("verbosity", pair->key))
-            {
-                enna_config_value_store(&value, "verbosity",
-                                        ENNA_CONFIG_STRING, pair);
-                enna_log(ENNA_MSG_INFO, NULL,
-                         " * verbosity level: %s", value);
-
-                if (!strcmp("verbose", value))
-                    verbosity = PLAYER_MSG_VERBOSE;
-                else if (!strcmp("info", value))
-                    verbosity = PLAYER_MSG_INFO;
-                else if (!strcmp("warning", value))
-                    verbosity = PLAYER_MSG_WARNING;
-                else if (!strcmp("error", value))
-                    verbosity = PLAYER_MSG_ERROR;
-                else if (!strcmp ("critical", value))
-                    verbosity = PLAYER_MSG_CRITICAL;
-                else if (!strcmp("none", value))
-                    verbosity = PLAYER_MSG_NONE;
-                else
-                    enna_log(ENNA_MSG_WARNING, NULL,
-                             "   - unknown verbosity, " \
-                             "'warning' used instead");
-            }
-
-            CFG_VIDEO(sub_align);
-            CFG_VIDEO(sub_pos);
-            CFG_VIDEO(sub_scale);
-            CFG_VIDEO(sub_visibility);
-            CFG_VIDEO(framedrop);
-        }
-    }
-
-    if (!value)
-        enna_log(ENNA_MSG_INFO, NULL,
-                 " * use all parameters by default");
+    memset(&param, 0, sizeof(player_init_param_t));
+    param.ao       = mp_cfg.ao;
+    param.vo       = mp_cfg.vo;
+    param.winid    = (unsigned long) enna->ee_winid;
+    param.event_cb = event_cb;
 
     /* Main player type is mandatory! */
-    if (!libplayer_wrapper_enabled(type))
+    if (!libplayer_wrapper_enabled(mp_cfg.type))
         goto err;
 
     /*
@@ -875,18 +984,20 @@ enna_mediaplayer_init(void)
      * Otherwise, if nothing is changed by the user but the wrapper is not
      * enabled, then the main type is used instead.
      */
-    if (dvd_type != type && !libplayer_wrapper_enabled(dvd_type))
+    if (mp_cfg.dvd_type != mp_cfg.type &&
+        !libplayer_wrapper_enabled(mp_cfg.dvd_type))
     {
-        if (dvd_set)
+        if (mp_cfg.dvd_set)
             goto err;
-        dvd_type = type;
+        mp_cfg.dvd_type = mp_cfg.type;
     }
 
-    if (tv_type != type && !libplayer_wrapper_enabled(tv_type))
+    if (mp_cfg.tv_type != mp_cfg.type &&
+        !libplayer_wrapper_enabled(mp_cfg.tv_type))
     {
-        if (tv_set)
+        if (mp_cfg.tv_set)
             goto err;
-        tv_type = type;
+        mp_cfg.tv_type = mp_cfg.type;
     }
 
     mp = calloc(1, sizeof(Enna_Mediaplayer));
@@ -902,21 +1013,46 @@ enna_mediaplayer_init(void)
 
     mp->pipe = ecore_pipe_add(pipe_read, NULL);
 
-    ENNA_MP_PLAYER_INIT(type);
-    if (dvd_type != type)
-        ENNA_MP_PLAYER_INIT(dvd_type);
-    if (tv_type != type && tv_type != dvd_type)
-        ENNA_MP_PLAYER_INIT(tv_type);
+    mp->players[mp_cfg.type] =
+        player_init(mp_cfg.type, mp_cfg.verbosity, &param);
+    if (!mp->players[mp_cfg.type])
+        goto err;
+    
+    if (mp_cfg.dvd_type != mp_cfg.type)
+    {
+        mp->players[mp_cfg.dvd_type] =
+            player_init(mp_cfg.dvd_type, mp_cfg.verbosity, &param);
+        if (!mp->players[mp_cfg.dvd_type])
+        {
+            enna_log(ENNA_MSG_WARNING, NULL,
+                     "DVD mediaplayer initialization failed, "
+                     "falling back to the default mediaplayer");
+            mp_cfg.dvd_type = mp_cfg.type;
+        }
+    }
+    
+    if (mp_cfg.tv_type != mp_cfg.type && mp_cfg.tv_type != mp_cfg.dvd_type)
+    {
+        mp->players[mp_cfg.tv_type] =
+            player_init(mp_cfg.tv_type, mp_cfg.verbosity, &param);
+        if (!mp->players[mp_cfg.tv_type])
+        {
+            enna_log(ENNA_MSG_WARNING, NULL,
+                     "TV mediaplayer initialization failed, "
+                     "falling back to the default mediaplayer");
+            mp_cfg.tv_type = mp_cfg.type;
+        }
+    }
 
     mp_event_cb_set(_event_cb, NULL);
 
     mp->uri = NULL;
     mp->label = NULL;
-    mp->default_type = type;
-    mp->dvd_type = dvd_type;
-    mp->tv_type = tv_type;
-    mp->player = mp->players[type];
-    mp->player_type = type;
+    mp->default_type = mp_cfg.type;
+    mp->dvd_type = mp_cfg.dvd_type;
+    mp->tv_type = mp_cfg.tv_type;
+    mp->player = mp->players[mp_cfg.type];
+    mp->player_type = mp_cfg.type;
     mp->play_state = STOPPED;
 
     /* Create Ecore Event ID */
@@ -966,7 +1102,7 @@ enna_mediaplayer_get_current_uri(Enna_Playlist *enna_playlist)
   list_item_t *item;
 
   item = eina_list_nth(enna_playlist->playlist, enna_playlist->selected);
-  if (!item->uri)
+  if (!item || !item->uri)
     return NULL;
   return strdup(item->uri);
 }
