@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -29,6 +29,7 @@
 #include "mediaplayer_obj.h"
 #include "utils.h"
 #include "logs.h"
+#include "xdg.h"
 
 #define SMART_NAME "mediaplayer_obj"
 
@@ -76,18 +77,18 @@ typedef struct _Button_Item
     void *data;
 }Button_Item;
 
-static int _start_cb(void *data, int type, void *event);
-static int _pause_cb(void *data, int type, void *event);
-static int _next_cb(void *data, int type, void *event);
-static int _prev_cb(void *data, int type, void *event);
-static int _unpause_cb(void *data, int type, void *event);
-static int _seek_cb(void *data, int type, void *event);
-static int _eos_cb(void *data, int type, void *event);
+static Eina_Bool _start_cb(void *data, int type, void *event);
+static Eina_Bool _pause_cb(void *data, int type, void *event);
+static Eina_Bool _next_cb(void *data, int type, void *event);
+static Eina_Bool _prev_cb(void *data, int type, void *event);
+static Eina_Bool _unpause_cb(void *data, int type, void *event);
+static Eina_Bool _seek_cb(void *data, int type, void *event);
+static Eina_Bool _eos_cb(void *data, int type, void *event);
 
 static void show_play_button(Smart_Data * sd);
 static void show_pause_button(Smart_Data * sd);
 
-static int _timer_cb(void *data);
+static Eina_Bool _timer_cb(void *data);
 
 #define METADATA_APPLY                                              \
     do                                                              \
@@ -153,8 +154,8 @@ _metadata_set(Evas_Object *obj, Enna_Metadata *metadata)
         if (*cover == '/')
             snprintf(cv, sizeof(cv), "%s", cover);
         else
-            snprintf(cv, sizeof(cv), "%s/.enna/covers/%s",
-                     enna_util_user_home_get(), cover);
+            snprintf(cv, sizeof(cv), "%s/covers/%s",
+                     enna_data_home_get(), cover);
 
         enna_image_file_set(sd->cv, cv, NULL);
     }
@@ -184,7 +185,7 @@ media_cover_hide (Smart_Data *sd)
 }
 
 /* Event from mediaplayer*/
-static int
+static Eina_Bool
 _start_cb(void *data, int type, void *event)
 {
     Smart_Data *sd = data;
@@ -201,7 +202,7 @@ _start_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _stop_cb(void *data, int type, void *event)
 {
     Smart_Data *sd = data;
@@ -215,7 +216,7 @@ _stop_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _prev_cb(void *data, int type, void *event)
 {
     Smart_Data *sd = data;
@@ -225,7 +226,7 @@ _prev_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _next_cb(void *data, int type, void *event)
 {
     Smart_Data *sd = data;
@@ -235,7 +236,7 @@ _next_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _unpause_cb(void *data, int type, void *event)
 {
     show_play_button(data);
@@ -243,7 +244,7 @@ _unpause_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _pause_cb(void *data, int type, void *event)
 {
     show_pause_button(data);
@@ -251,7 +252,7 @@ _pause_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _seek_cb(void *data, int type, void *event)
 {
     Enna_Event_Mediaplayer_Seek_Data *ev;
@@ -261,7 +262,7 @@ _seek_cb(void *data, int type, void *event)
     return 1;
 }
 
-static int
+static Eina_Bool
 _eos_cb(void *data, int type, void *event)
 {
     Smart_Data *sd = data;
@@ -300,7 +301,7 @@ slider_position_update(Smart_Data *sd)
 }
 
 /* Update position Timer callback */
-static int
+static Eina_Bool
 _timer_cb(void *data)
 {
     Smart_Data *sd = data;
@@ -311,7 +312,7 @@ _timer_cb(void *data)
         slider_position_update(sd);
     }
 
-    return 1;
+    return EINA_TRUE;
 }
 
 /* events from buttons*/
@@ -586,6 +587,7 @@ enna_mediaplayer_obj_add(Evas * evas, Enna_Playlist *enna_playlist)
     sd->text_box = bx;
 
     lb = elm_label_add(layout);
+    elm_object_style_set(lb, "enna");
     evas_object_size_hint_weight_set(lb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
     evas_object_size_hint_align_set(lb, 0.5, 0.5);
     elm_label_label_set(lb, "");
@@ -594,6 +596,7 @@ enna_mediaplayer_obj_add(Evas * evas, Enna_Playlist *enna_playlist)
     sd->title = lb;
 
     lb = elm_label_add(layout);
+    elm_object_style_set(lb, "enna");
     evas_object_size_hint_weight_set(lb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
     evas_object_size_hint_align_set(lb, 0.5, 0.5);
     elm_label_label_set(lb, "");
@@ -602,6 +605,7 @@ enna_mediaplayer_obj_add(Evas * evas, Enna_Playlist *enna_playlist)
     sd->album = lb;
 
     lb = elm_label_add(layout);
+    elm_object_style_set(lb, "enna");
     evas_object_size_hint_weight_set(lb, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
     evas_object_size_hint_align_set(lb, 0.5, 0.5);
     elm_label_label_set(lb, "");
@@ -685,6 +689,13 @@ enna_mediaplayer_obj_layout_set(Evas_Object *obj, const char *layout)
     edje_object_signal_emit(elm_layout_edje_get(sd->layout), layout, "enna");
 }
 
+void
+enna_mediaplayer_obj_metadata_refresh(Evas_Object *obj)
+{
+    Smart_Data *sd = evas_object_data_get(obj, "sd");
+    METADATA_APPLY;
+}
+
 void
 enna_mediaplayer_position_update(Evas_Object *obj)
 {
