@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -22,18 +22,31 @@
 #ifndef METADATA_H
 #define METADATA_H
 
-typedef void * Enna_Metadata;
+#include "vfs.h"
 
+typedef struct _Enna_Metadata Enna_Metadata;
+
+typedef enum _Enna_Metadata_OnDemand
+{
+    ENNA_METADATA_OD_PARSED,
+    ENNA_METADATA_OD_GRABBED,
+    ENNA_METADATA_OD_ENDED,
+} Enna_Metadata_OnDemand;
+
+void enna_metadata_cfg_register(void);
 void enna_metadata_init(void);
 void enna_metadata_shutdown(void);
 void *enna_metadata_get_db(void);
 
 Enna_Metadata *enna_metadata_meta_new(const char *file);
-char *enna_metadata_meta_get(Enna_Metadata *meta, const char *name, int max);
-char *enna_metadata_meta_get_all(Enna_Metadata *meta);
+char *enna_metadata_meta_get(const Enna_Metadata *meta,
+                             const char *name, int max);
+char *enna_metadata_meta_get_all(const Enna_Metadata *meta);
 void  enna_metadata_meta_free(Enna_Metadata *meta);
 void enna_metadata_set_position(Enna_Metadata *meta, double position);
-void enna_metadata_ondemand(const char *file);
-char *enna_metadata_meta_duration_get(Enna_Metadata *m);
+void enna_metadata_ondemand(const Enna_Vfs_File *file,
+                            void (*odcb)(const Enna_Vfs_File *file,
+                                         Enna_Metadata_OnDemand ev));
+char *enna_metadata_meta_duration_get(const Enna_Metadata *m);
 
 #endif /* METADATA_H */
