@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -23,7 +23,6 @@
 
 #include <Ecore.h>
 #include <Ecore_File.h>
-#include <Ecore_Data.h>
 #include <Edje.h>
 #include <Elementary.h>
 
@@ -667,7 +666,8 @@ _smart_add(Evas_Object * obj)
         return;
     sd->obj = obj;
     sd->nb = -1;
-    sd->o_scroll = elm_scroller_add(obj);
+    sd->o_scroll = elm_scroller_add(enna->layout);
+    elm_object_style_set(sd->o_scroll, "enna");
 
     elm_scroller_policy_set(sd->o_scroll, ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_OFF);
     elm_scroller_bounce_set(sd->o_scroll, 0, 0);
