@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -22,6 +22,6 @@
 #ifndef BOOKSTORE_GOCOMICS_H
 #define BOOKSTORE_GOCOMICS_H
 
-BookStore_Service bs_gocomics;
+Bookstore_Service bs_gocomics;
 
 #endif /* BOOKSTORE_GOCOMICS_H */
