@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -22,7 +22,7 @@
 #ifndef PHOTO_SLIDESHOW_VIEW_H
 #define PHOTO_SLIDESHOW_VIEW_H
 
-Evas_Object *enna_photo_slideshow_add(Evas * evas);
+Evas_Object *enna_photo_slideshow_add(Evas_Object *parent);
 void enna_photo_slideshow_next(Evas_Object *obj);
 void enna_photo_slideshow_previous(Evas_Object *obj);
 int enna_photo_slideshow_timeout_get(Evas_Object *obj);
