@@ -1,6 +1,6 @@
 /*
  * GeeXboX Enna Media Center.
- * Copyright (C) 2005-2009 The Enna Project
+ * Copyright (C) 2005-2010 The Enna Project
  *
  * This file is part of Enna.
  *
@@ -48,6 +48,7 @@
 #include "metadata.h"
 #include "utils.h"
 #include "mediaplayer_obj.h"
+#include "xdg.h"
 
 #include "video.h"
 #include "video_flags.h"
@@ -69,7 +70,7 @@ static void browse(void *data);
 static void _create_menu(void);
 static void _return_to_video_info_gui();
 
-static int _eos_cb(void *data, int type, void *event);
+static Eina_Bool _eos_cb(void *data, int type, void *event);
 static void video_infos_del(void);
 
 typedef struct _Enna_Module_Video Enna_Module_Video;
@@ -103,6 +104,7 @@ struct _Enna_Module_Video
     int infos_displayed;
     int resume_displayed;
     int controls_displayed;
+    char *uri_hilighted;
     Enna_Volumes_Listener *vl;
     Ecore_Timer *controls_timer;
     Ecore_Event_Handler *mouse_button_event_handler;
@@ -133,7 +135,7 @@ end:
     edje_object_part_text_set(mod->o_edje, "movies.counter.label", label);
 }
 
-static int
+static Eina_Bool
 _controls_timer_cb(void *data)
 {
     media_controls_display(0);
@@ -144,14 +146,16 @@ _controls_timer_cb(void *data)
 static void
 video_resize(void)
 {
+
     Evas_Coord w, h, x, y;
     Evas_Coord h2 = 0;
-
+#ifndef BUILD_BACKEND_EMOTION
     if (mod->controls_displayed)
         evas_object_geometry_get(mod->o_mediacontrols, NULL, NULL, NULL, &h2);
-
+#endif
     evas_object_geometry_get(mod->o_mediaplayer, &x, &y, &w, &h);
     enna_mediaplayer_video_resize(x, y, w, h - h2);
+
 }
 
 void
@@ -168,6 +172,8 @@ media_controls_display(int show)
         return;
 
     mod->controls_displayed = show;
+    show ? evas_object_show(mod->o_mediacontrols) : evas_object_hide(mod->o_mediacontrols);
+
     video_resize();
 }
 
@@ -176,7 +182,7 @@ menu_view_event(enna_input event)
 {
     switch (event)
     {
-    case ENNA_INPUT_EXIT:
+    case ENNA_INPUT_BACK:
         enna_content_hide();
         enna_mainmenu_show();
         break;
@@ -204,7 +210,7 @@ videoplayer_view_event_no_display (enna_input event)
     case ENNA_INPUT_OK:
         media_controls_display(!mod->controls_displayed);
         break;
-    case ENNA_INPUT_SPACE:
+    case ENNA_INPUT_PLAY:
         enna_mediaplayer_play(mod->enna_playlist);
         break;
     case ENNA_INPUT_RIGHT:
@@ -219,58 +225,58 @@ videoplayer_view_event_no_display (enna_input event)
     case ENNA_INPUT_DOWN:
         _seek_video(-60); /* -60s */
         break;
-    case ENNA_INPUT_PREV:
+    case ENNA_INPUT_FORWARD:
         _seek_video(+600); /* +10min */
         break;
-    case ENNA_INPUT_NEXT:
+    case ENNA_INPUT_REWIND:
         _seek_video(-600); /* -10min */
         break;
-    case ENNA_INPUT_KEY_M:
+    case ENNA_INPUT_MUTE:
         enna_mediaplayer_mute();
         break;
-    case ENNA_INPUT_KEY_K:
+    case ENNA_INPUT_AUDIO_PREV:
         enna_mediaplayer_audio_previous();
         break;
-    case ENNA_INPUT_KEY_L:
+    case ENNA_INPUT_AUDIO_NEXT:
         enna_mediaplayer_audio_next();
         break;
-    case ENNA_INPUT_KEY_P:
+    case ENNA_INPUT_AUDIO_DELAY_PLUS:
         enna_mediaplayer_audio_increase_delay();
         break;
-    case ENNA_INPUT_KEY_O:
+    case ENNA_INPUT_AUDIO_DELAY_MINUS:
         enna_mediaplayer_audio_decrease_delay();
         break;
-    case ENNA_INPUT_KEY_S:
+    case ENNA_INPUT_SUBTITLES:
         enna_mediaplayer_subtitle_set_visibility();
         break;
-    case ENNA_INPUT_KEY_G:
+    case ENNA_INPUT_SUBS_PREV:
         enna_mediaplayer_subtitle_previous();
         break;
-    case ENNA_INPUT_KEY_Y:
+    case ENNA_INPUT_SUBS_NEXT:
         enna_mediaplayer_subtitle_next();
         break;
-    case ENNA_INPUT_KEY_A:
+    case ENNA_INPUT_SUBS_ALIGN:
         enna_mediaplayer_subtitle_set_alignment();
         break;
-    case ENNA_INPUT_KEY_T:
+    case ENNA_INPUT_SUBS_POS_PLUS:
         enna_mediaplayer_subtitle_increase_position();
         break;
-    case ENNA_INPUT_KEY_R:
+    case ENNA_INPUT_SUBS_POS_MINUS:
         enna_mediaplayer_subtitle_decrease_position();
         break;
-    case ENNA_INPUT_KEY_J:
+    case ENNA_INPUT_SUBS_SCALE_PLUS:
         enna_mediaplayer_subtitle_increase_scale();
         break;
-    case ENNA_INPUT_KEY_I:
+    case ENNA_INPUT_SUBS_SCALE_MINUS:
         enna_mediaplayer_subtitle_decrease_scale();
         break;
-    case ENNA_INPUT_KEY_X:
+    case ENNA_INPUT_SUBS_DELAY_PLUS:
         enna_mediaplayer_subtitle_increase_delay();
         break;
-    case ENNA_INPUT_KEY_Z:
+    case ENNA_INPUT_SUBS_DELAY_MINUS:
         enna_mediaplayer_subtitle_decrease_delay();
         break;
-    case ENNA_INPUT_KEY_W:
+    case ENNA_INPUT_FRAMEDROP:
         enna_mediaplayer_set_framedrop();
         break;
     default:
@@ -284,7 +290,8 @@ videoplayer_view_event(enna_input event)
     switch (event)
     {
     case ENNA_INPUT_QUIT:
-    case ENNA_INPUT_EXIT:
+    case ENNA_INPUT_BACK:
+    case ENNA_INPUT_STOP:
         _return_to_video_info_gui();
         break;
     default:
@@ -325,10 +332,10 @@ _return_to_video_info_gui()
     pos = enna_mediaplayer_position_get();
     enna_metadata_set_position(m, pos);
     enna_mediaplayer_stop();
-    mod->state = BROWSER_VIEW;
+    mod->state = MENU_VIEW;
 }
 
-static int
+static Eina_Bool
 _eos_cb(void *data, int type, void *event)
 {
     _return_to_video_info_gui();
@@ -354,8 +361,8 @@ backdrop_show(Enna_Metadata *m)
         if (*backdrop == '/')
             snprintf(dst, sizeof (dst), "%s", backdrop);
         else
-            snprintf(dst, sizeof (dst), "%s/.enna/fanarts/%s",
-                     enna_util_user_home_get (), backdrop);
+            snprintf(dst, sizeof (dst), "%s/fanarts/%s",
+                     enna_data_home_get(), backdrop);
         file = strdup(dst);
 
         enna_video_picture_set(mod->o_backdrop, file, from_vfs);
@@ -380,7 +387,6 @@ snapshot_show(Enna_Metadata *m, int dir)
     int from_vfs = 1;
     char *snapshot;
 
-    enna_video_picture_unset(mod->o_snapshot);
     snapshot = enna_metadata_meta_get(m, "fanart", 1);
     if (snapshot)
     {
@@ -389,8 +395,8 @@ snapshot_show(Enna_Metadata *m, int dir)
         if (*snapshot == '/')
             snprintf(dst, sizeof(dst), "%s", snapshot);
         else
-            snprintf(dst, sizeof(dst), "%s/.enna/fanarts/%s",
-                     enna_util_user_home_get(), snapshot);
+            snprintf(dst, sizeof(dst), "%s/fanarts/%s",
+                     enna_data_home_get(), snapshot);
         file = strdup(dst);
     }
     else
@@ -433,20 +439,20 @@ browser_view_event(enna_input event)
     /* handle resume popup, if any */
     if (mod->resume_displayed)
     {
-        if (event == ENNA_INPUT_EXIT)
+        if (event == ENNA_INPUT_BACK)
             popup_resume_display(0);
         else
             video_resume_input_feed(mod->o_resume, event);
         return;
     }
 
-    if (event == ENNA_INPUT_KEY_I)
+    if (event == ENNA_INPUT_INFO)
     {
         panel_infos_display(!mod->infos_displayed);
         return;
     }
 
-    if (event == ENNA_INPUT_EXIT)
+    if (event == ENNA_INPUT_BACK)
     {
         video_infos_del();
         update_movies_counter(NULL);
@@ -493,7 +499,7 @@ _mediaplayer_mouse_up_cb(void *data,
     media_controls_display(1);
 }
 
-static int
+static Eina_Bool
 _mediaplayer_mouse_move_libplayer_cb(void *data, int type, void *event)
 {
     Ecore_Event_Mouse_Move *e = event;
@@ -509,7 +515,7 @@ _mediaplayer_mouse_move_libplayer_cb(void *data, int type, void *event)
     return 0;
 }
 
-static int
+static Eina_Bool
 _mediaplayer_mouse_down_libplayer_cb(void *data, int type, void *event)
 {
     Ecore_Event_Mouse_Button *e = event;
@@ -528,6 +534,9 @@ _mediaplayer_mouse_down_libplayer_cb(void *data, int type, void *event)
 void
 movie_start_playback(int resume)
 {
+    const Evas_Object *ed;
+    Evas_Coord x, y, w, h;
+
     panel_infos_display(0);
     mod->state = VIDEOPLAYER_VIEW;
 
@@ -545,8 +554,15 @@ movie_start_playback(int resume)
 
     mod->o_mediacontrols =
         enna_mediaplayer_obj_add(enna->evas, mod->enna_playlist);
-    edje_object_part_swallow(mod->o_edje,
-                             "controls.swallow", mod->o_mediacontrols);
+    
+    ed = edje_object_part_object_get(mod->o_edje, "controls.swallow");
+    evas_object_geometry_get(ed, &x, &y, &w, &h);
+    evas_object_move(mod->o_mediacontrols, x, y);
+    evas_object_resize(mod->o_mediacontrols, w, h);
+    evas_object_show(mod->o_mediacontrols);
+
+    /*edje_object_part_swallow(mod->o_edje,
+                             "controls.swallow", mod->o_mediacontrols);*/
     enna_mediaplayer_obj_layout_set(mod->o_mediacontrols, "layout,video");
 
     mod->mouse_button_event_handler =
@@ -555,7 +571,7 @@ movie_start_playback(int resume)
     mod->mouse_move_event_handler =
         ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE,
                                 _mediaplayer_mouse_move_libplayer_cb, NULL);
-
+#ifdef BUILD_BACKEND_LIBPLAYER
     evas_object_event_callback_add(mod->o_mediacontrols,
                                    EVAS_CALLBACK_MOUSE_MOVE,
                                    _mediaplayer_mouse_move_cb, NULL);
@@ -565,7 +581,17 @@ movie_start_playback(int resume)
     evas_object_event_callback_add(mod->o_mediacontrols,
                                    EVAS_CALLBACK_RESIZE,
                                    _mediaplayer_resize_cb, NULL);
-
+#else
+    evas_object_event_callback_add(enna_mediaplayer_obj_get(),
+                                   EVAS_CALLBACK_MOUSE_MOVE,
+                                   _mediaplayer_mouse_move_cb, NULL);
+    evas_object_event_callback_add(enna_mediaplayer_obj_get(),
+                                   EVAS_CALLBACK_MOUSE_UP,
+                                   _mediaplayer_mouse_up_cb, NULL);
+    evas_object_event_callback_add(enna_mediaplayer_obj_get(),
+                                   EVAS_CALLBACK_RESIZE,
+                                   _mediaplayer_resize_cb, NULL);
+#endif
     enna_mediaplayer_stop();
     enna_mediaplayer_obj_event_catch(mod->o_mediacontrols);
     enna_mediaplayer_play(mod->enna_playlist);
@@ -643,62 +669,81 @@ browser_cb_select(void *data, Evas_Object *obj, void *event_info)
     free(ev);
 }
 
+static void
+video_infos_display_title(const Enna_Vfs_File *file, const Enna_Metadata *m)
+{
+    char *title;
+    const char *label;
+
+    title = enna_metadata_meta_get(m, "title", 1);
+    label = title ? title : file->label;
+    edje_object_part_text_set(mod->o_edje, "title.label", label);
+
+    free(title);
+}
 
 static void
-video_infos_display(Enna_Vfs_File *file, int delay)
+video_infos_display_genre(const Enna_Vfs_File *file, const Enna_Metadata *m)
+{
+    char *categories;
+
+    categories = enna_metadata_meta_get(m, "category", 5);
+    edje_object_part_text_set(mod->o_edje, "genre.label",
+                              categories ? categories : "");
+
+    free(categories);
+}
+
+static void
+video_infos_display_length(const Enna_Vfs_File *file, const Enna_Metadata *m)
+{
+    char *length;
+
+    length = enna_metadata_meta_duration_get(m);
+    edje_object_part_text_set(mod->o_edje, "length.label",
+                              length ? length : "");
+
+    free(length);
+}
+
+static void
+video_infos_display_synopsis(const Enna_Vfs_File *file, const Enna_Metadata *m)
+{
+    char *synopsis;
+
+    synopsis = enna_metadata_meta_get(m, "synopsis", 1);
+    edje_object_part_text_set(mod->o_edje, "synopsis.textblock",
+                              synopsis ? synopsis : "");
+    edje_object_signal_emit(mod->o_edje, synopsis ?
+                            "separator,show" : "separator,hide", "enna");
+
+    free(synopsis);
+}
+
+static void
+video_infos_display(const Enna_Vfs_File *file)
 {
     Enna_Metadata *m;
 
     if (!file)
         return;
 
+    /* If m is NULL, the panels will be cleaned. */
     m = enna_metadata_meta_new(file->uri);
-    if (!m)
-        return;
 
-    if (!delay)
-    {
-        char *title, *categories, *length, *synopsis;
-        const char *label;
+    video_infos_display_title(file, m);
+    video_infos_display_genre(file, m);
+    video_infos_display_length(file, m);
+    video_infos_display_synopsis(file, m);
 
-        /* title */
-        title = enna_metadata_meta_get(m, "title", 1);
-        label = title ? title : file->label;
-        edje_object_part_text_set(mod->o_edje, "title.label", label);
+    backdrop_show(m);
+    snapshot_show(m, file->is_directory);
 
-        /* genre */
-        categories = enna_metadata_meta_get(m, "category", 5);
-        edje_object_part_text_set(mod->o_edje, "genre.label",
-                                  categories ? categories : "");
+    enna_video_flags_update(mod->o_video_flags, m);
 
-        /* length */
-        length = enna_metadata_meta_duration_get(m);
-        edje_object_part_text_set(mod->o_edje, "length.label",
-                                  length ? length : "");
-
-        /* synopsis */
-        synopsis = enna_metadata_meta_get(m, "synopsis", 1);
-        edje_object_part_text_set(mod->o_edje, "synopsis.textblock",
-                                  synopsis ? synopsis : "");
-        edje_object_signal_emit(mod->o_edje, synopsis ?
-                                "separator,show" : "separator,hide", "enna");
-
-        ENNA_FREE(title);
-        ENNA_FREE(categories);
-        ENNA_FREE(length);
-        ENNA_FREE(synopsis);
-    }
-    else
-    {
-        backdrop_show(m);
-        snapshot_show(m, file->is_directory);
-
-        enna_video_flags_update(mod->o_video_flags, m);
-
-        enna_panel_infos_set_cover(mod->o_panel_infos, m);
-        enna_panel_infos_set_text(mod->o_panel_infos, m);
-        enna_panel_infos_set_rating(mod->o_panel_infos, m);
-    }
+    enna_panel_infos_set_cover(mod->o_panel_infos, m);
+    enna_panel_infos_set_text(mod->o_panel_infos, m);
+    enna_panel_infos_set_rating(mod->o_panel_infos, m);
 
     enna_metadata_meta_free(m);
 }
@@ -718,6 +763,50 @@ video_infos_del (void)
     edje_object_signal_emit(mod->o_edje, "separator,hide", "enna");
 }
 
+static void
+_ondemand_cb_refresh(const Enna_Vfs_File *file, Enna_Metadata_OnDemand ev)
+{
+    Enna_Metadata *m;
+
+    if (!file || !file->uri || !mod->uri_hilighted)
+        return;
+
+    if (file->is_directory || file->is_menu)
+        return;
+
+    if (strcmp(file->uri, mod->uri_hilighted))
+        return;
+
+    m = enna_metadata_meta_new(file->uri);
+    if (!m)
+        return;
+
+    switch (ev)
+    {
+    case ENNA_METADATA_OD_PARSED:
+    case ENNA_METADATA_OD_GRABBED:
+        video_infos_display_title(file, m);
+        video_infos_display_genre(file, m);
+        video_infos_display_length(file, m);
+        video_infos_display_synopsis(file, m);
+        enna_video_flags_update(mod->o_video_flags, m);
+        enna_panel_infos_set_text(mod->o_panel_infos, m);
+        enna_panel_infos_set_rating(mod->o_panel_infos, m);
+        if (ev == ENNA_METADATA_OD_PARSED)
+            break;
+    case ENNA_METADATA_OD_ENDED:
+        backdrop_show(m);
+        snapshot_show(m, file->is_directory);
+        enna_panel_infos_set_cover(mod->o_panel_infos, m);
+        break;
+
+    default:
+        break;
+    }
+
+    enna_metadata_meta_free(m);
+}
+
 static void
 browser_cb_delay_hilight(void *data, Evas_Object *obj, void *event_info)
 {
@@ -727,13 +816,12 @@ browser_cb_delay_hilight(void *data, Evas_Object *obj, void *event_info)
         return;
 
     if (!ev->file->is_directory && !ev->file->is_menu)
-    {
         /* ask for on-demand scan for local files */
         if (!strncmp(ev->file->uri, "file://", 7))
-            enna_metadata_ondemand(ev->file->uri + 7);
+            enna_metadata_ondemand(ev->file, _ondemand_cb_refresh);
 
-        video_infos_display(ev->file, 1);
-    }
+    ENNA_FREE(mod->uri_hilighted);
+    mod->uri_hilighted = strdup(ev->file->uri);
 }
 
 static void
@@ -745,7 +833,10 @@ browser_cb_hilight(void *data, Evas_Object *obj, void *event_info)
         return;
 
     if (!ev->file->is_directory && !ev->file->is_menu)
-        video_infos_display(ev->file, 0);
+        video_infos_display(ev->file);
+
+    ENNA_FREE(mod->uri_hilighted);
+    mod->uri_hilighted = strdup(ev->file->uri);
 }
 
 static void
@@ -885,6 +976,7 @@ _class_show(int dummy)
 static void
 _class_hide(int dummy)
 {
+    _return_to_video_info_gui();
     edje_object_signal_emit(mod->o_edje, "module,hide", "enna");
 }
 
@@ -941,14 +1033,14 @@ em_init(Enna_Module *em)
     mod->o_snapshot = enna_video_picture_add(enna->evas);
     mod->eos_event_handler =
         ecore_event_handler_add(ENNA_EVENT_MEDIAPLAYER_EOS, _eos_cb, NULL);
-    enna_activity_add(&class);
+    enna_activity_register(&class);
     mod->enna_playlist = enna_mediaplayer_playlist_create();
 }
 
 static void
 em_shutdown(Enna_Module *em)
 {
-    enna_activity_del(ENNA_MODULE_NAME);
+    enna_activity_unregister(&class);
     ENNA_EVENT_HANDLER_DEL(mod->eos_event_handler);
     ENNA_OBJECT_DEL(mod->o_edje);
     ENNA_OBJECT_DEL(mod->o_list);
@@ -970,6 +1062,7 @@ em_shutdown(Enna_Module *em)
     ENNA_OBJECT_DEL(mod->o_resume);
     ENNA_OBJECT_DEL(mod->o_video_flags);
     ENNA_FREE(mod->o_current_uri);
+    ENNA_FREE(mod->uri_hilighted);
     enna_mediaplayer_playlist_free(mod->enna_playlist);
     free(mod);
 }
@@ -983,18 +1076,8 @@ em_shutdown(Enna_Module *em)
 #define MOD_PREFIX enna_mod_activity_video
 #endif /* USE_STATIC_MODULES */
 
-Enna_Module_Api ENNA_MODULE_API =
-{
-    ENNA_MODULE_VERSION,
-    "activity_video",
-    N_("Video"),
-    "icon/video",
-    N_("This is the main video module"),
-    "bla bla bla<br><b>bla bla bla</b><br><br>bla."
-};
-
-void
-ENNA_MODULE_INIT(Enna_Module *em)
+static void
+module_init(Enna_Module *em)
 {
     if (!em)
         return;
@@ -1002,8 +1085,23 @@ ENNA_MODULE_INIT(Enna_Module *em)
     em_init(em);
 }
 
-void
-ENNA_MODULE_SHUTDOWN(Enna_Module *em)
+static void
+module_shutdown(Enna_Module *em)
 {
     em_shutdown(em);
 }
+
+Enna_Module_Api ENNA_MODULE_API =
+{
+    ENNA_MODULE_VERSION,
+    "activity_video",
+    N_("Video"),
+    "icon/video",
+    N_("Play your videos"),
+    "bla bla bla<br><b>bla bla bla</b><br><br>bla.",
+    {
+        module_init,
+        module_shutdown
+    }
+};
+
